/*
 * FromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef FROM_XML_PARSER_H_
#define FROM_XML_PARSER_H_

#include "sax/FromXMLParserHelper.h"
#include <list>
#include <set>
#include "sax/Token.h"
#include "exception/AlibException.h"

namespace alib {

/**
 * Parser used to get String from XML parsed into list of tokens.
 */
template<class T, class F>
class FromXMLParser : public sax::FromXMLParserHelper {
	virtual T parse(std::list<sax::Token>& input) const = 0;
	virtual T parse(std::list<sax::Token>& input, const std::set<F>& features) const = 0;

public:
	virtual bool first(std::list<sax::Token>& input) const = 0;

	/**
	 * Parses the XML tokens and returns symbol. The input is destroyed in the process.
	 * @param input XML tokens represented as list of tokens
	 * @return Symbol* the parsed symbol or NULL when by first token it is not a symbol
	 * @throws ParserException when tokens do not represent Symbol but first token seemd as a symbol
	 */
	T* parsePointer(std::list<sax::Token>& input) const {
		if(first(input)) {
			return new T(parse(input));
		} else {
			return NULL;
		}
	}
	
	/**
	 * Parses the XML tokens and returns symbol. The input is destroyed in the process.
	 * @param input XML tokens represented as list of tokens
	 * @return Symbol the parsed symbol
	 * @throws ParserException when tokens do not represent Symbol
	 */
	T parseValue(std::list<sax::Token>& input) const {
		T res = parse(input);

		if(input.size() == 0) {
			return res;
		} else {
			throw exception::AlibException();
		}
	}

};

} /* namespace alib */

#endif /* FROM_XML_PARSER_HELPER_H_ */
