/*
 * Automaton.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Martin Zak
 */

#include "Automaton.h"

namespace automaton {

Automaton::Automaton(const AutomatonBase& automaton) : automaton(automaton.clone()) {

}

Automaton::Automaton(AutomatonBase&& automaton) : automaton(std::move(automaton).plunder()) {

}

Automaton::Automaton(const Automaton& other) : automaton(other.getAutomaton().clone()) {

}

Automaton::Automaton(Automaton&& other) noexcept : automaton(std::move(other.getAutomaton()).plunder()) {
	other.automaton = NULL;
}

Automaton& Automaton::operator=(const Automaton& other) {
	if(this == &other) return *this;

	delete automaton;
	automaton = other.getAutomaton().clone();

	return *this;
}

Automaton& Automaton::operator=(Automaton&& other) noexcept {
	std::swap(this->automaton, other.automaton);
	return *this;
}

Automaton::~Automaton() {
	delete automaton;
}

const AutomatonBase& Automaton::getAutomaton() const {
	return *automaton;
}

AutomatonBase& Automaton::getAutomaton() {
	return *automaton;
}

void Automaton::setAutomaton(const AutomatonBase& automaton) {
	delete this->automaton;
	this->automaton = automaton.clone();
}

void Automaton::setAutomaton(AutomatonBase&& automaton) {
	delete this->automaton;
	this->automaton = std::move(automaton).plunder();
}

bool Automaton::operator!=(const Automaton& other) const {
	return !(*this == other);
}

bool Automaton::operator==(const Automaton& other) const {
	return this->getAutomaton() == other.getAutomaton();
}

std::ostream& operator<<(std::ostream& os, const Automaton& automaton) {
	os << automaton.getAutomaton();
	return os;
}

} /* namespace automaton */

