/*
 * AutomatonBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_BASE_H_
#define AUTOMATON_BASE_H_

#include "../std/visitor.hpp"
#include <iostream>

namespace automaton {

class UnknownAutomaton;
class DFA;
class NFA;
class EpsilonNFA;
class CompactNFA;
class ExtendedNFA;
class PDA;
class OneTapeDTM;

/**
 * Abstract base class for all automata.
 */
class AutomatonBase : public std::elementBase<UnknownAutomaton, DFA, NFA, EpsilonNFA, CompactNFA, ExtendedNFA, PDA, OneTapeDTM> {
public:
	virtual AutomatonBase* clone() const = 0;

	virtual AutomatonBase* plunder() && = 0;

	virtual ~AutomatonBase() noexcept;

	virtual bool operator!=(const AutomatonBase& other) const;

	virtual bool operator==(const UnknownAutomaton& other) const;

	virtual bool operator==(const DFA& other) const;

	virtual bool operator==(const NFA& other) const;

	virtual bool operator==(const EpsilonNFA& other) const;

	virtual bool operator==(const CompactNFA& other) const;

	virtual bool operator==(const ExtendedNFA& other) const;

	virtual bool operator==(const PDA& other) const;

	virtual bool operator==(const OneTapeDTM& other) const;

	virtual bool operator==(const AutomatonBase& other) const = 0;

	friend std::ostream& operator<<(std::ostream& os, const AutomatonBase& automaton);

	virtual void operator>>(std::ostream&) const = 0;
};

} /* namespace automaton */

#endif /* AUTOMATON_BASE_H_ */

