/*
 * AutomatonFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef AUTOMATON_FROM_XML_PARSER_H_
#define AUTOMATON_FROM_XML_PARSER_H_

#include "../FromXMLParser.hpp"
#include "Automaton.h"
#include "AutomatonFeatures.h"
#include "UnknownAutomaton.h"
#include "FSM/EpsilonNFA.h"
#include "FSM/NFA.h"
#include "FSM/DFA.h"

#include <list>
#include <set>
#include "../std/variant.hpp"
#include "../sax/Token.h"

namespace automaton {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into list of Tokens.
 */
class AutomatonFromXMLParser : public alib::FromXMLParser<Automaton, FEATURES> {
protected:
	std::string parseLabel(std::list<sax::Token> &input) const;

	std::set<State> parseStates(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseInputAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseStackAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseInitialStackSymbols(std::list<sax::Token> &input) const;
	alphabet::Symbol parseInitialStackSymbol(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseTapeAlphabet(std::list<sax::Token> &input) const;
	State parseInitialState(std::list<sax::Token> &input) const;
	std::set<State> parseInitialStates(std::list<sax::Token> &input) const;
	std::set<State> parseFinalStates(std::list<sax::Token> &input) const;
	alphabet::Symbol parseBlankSymbol(std::list<sax::Token> &input) const;

	State parseTransitionFrom(std::list<sax::Token>& input) const;
	State parseTransitionTo(std::list<sax::Token>& input) const;
	Shift parseTransitionShift(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseTransitionPop(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseTransitionPush(std::list<sax::Token>& input) const;

	template<class T>
	void parseTransitions(std::list<sax::Token> &input, T& automaton) const;
	void parseTransition(std::list<sax::Token>& input, UnknownAutomaton& automaton) const;
	void parseTransition(std::list<sax::Token>& input, EpsilonNFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, NFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, DFA& automaton) const;

	alphabet::Symbol parseTransitionInputSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionInputEpsilonSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionInputBlankEpsilonSymbol(std::list<sax::Token> &input) const;
	alphabet::Symbol parseTransitionOutputSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionOutputEpsilonSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionOutputBlankEpsilonSymbol(std::list<sax::Token> &input) const;

	alphabet::Symbol parseSymbolIO(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseEpsilonSymbolIO(std::list<sax::Token> &input) const;

	virtual Automaton parse(std::list<sax::Token>& input) const;
	virtual Automaton parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;
public:
	virtual bool first(std::list<sax::Token>& input) const;

	UnknownAutomaton parseUnknownAutomaton(std::list<sax::Token>& input) const;
	EpsilonNFA parseEpsilonNFA(std::list<sax::Token>& input) const;
	NFA parseNFA(std::list<sax::Token>& input) const;
	DFA parseDFA(std::list<sax::Token>& input) const;
};

} /* namespace automaton */

#endif /* AUTOMATON_FROM_XML_PARSER_H_ */

