/*
 * AutomatonToXMLComposer.h
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_TO_XML_COMPOSER_H_
#define AUTOMATON_TO_XML_COMPOSER_H_

#include <string>
#include <list>
#include "Automaton.h"
#include "UnknownAutomaton.h"
#include "FSM/EpsilonNFA.h"
#include "FSM/NFA.h"
#include "FSM/DFA.h"
#include "../sax/Token.h"

namespace automaton {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class AutomatonToXMLComposer : public AutomatonBase::const_visitor_type {
	void Visit(void*, const Automaton& automaton) const;
	void Visit(void*, const UnknownAutomaton& automaton) const;
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const PDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

protected:
	void composeStates(std::list<sax::Token>&, const std::set<State>& states) const;
	void composeInputAlphabet(std::list<sax::Token>&, const std::set<alphabet::Symbol>& symbols) const;
	void composeInitialStates(std::list<sax::Token>&, const std::set<State>& states) const;
	void composeInitialState(std::list<sax::Token>&, const State& state) const;
	void composeFinalStates(std::list<sax::Token>&, const std::set<State>& states) const;
	void composeStackAlphabet(std::list<sax::Token>&, const std::set<alphabet::Symbol>& symbols) const;
	void composeInitialStackSymbols(std::list<sax::Token>&, const std::set<alphabet::Symbol>& symbols) const;
	void composeInitialStackSymbol(std::list<sax::Token>&, const alphabet::Symbol& symbols) const;
	void composeTapeAlphabet(std::list<sax::Token>&, const std::set<alphabet::Symbol>& symbols) const;
	void composeBlankSymbol(std::list<sax::Token>&, const alphabet::Symbol& symbol) const;

	void composeTransitions(std::list<sax::Token>&, const UnknownAutomaton& automaton) const;
	void composeTransitions(std::list<sax::Token>&, const EpsilonNFA& automaton) const;
	void composeTransitions(std::list<sax::Token>&, const NFA& automaton) const;
	void composeTransitions(std::list<sax::Token>&, const DFA& automaton) const;

	void composeTransitionTo(std::list<sax::Token>&, const State& state) const;
	void composeTransitionFrom(std::list<sax::Token>&, const State& state) const;
	void composeTransitionShift(std::list<sax::Token>&, const Shift shift) const;
	void composeTransitionPop(std::list<sax::Token>&, const std::vector<alphabet::Symbol>& symbols) const;
	void composeTransitionPush(std::list<sax::Token>&, const std::vector<alphabet::Symbol>& symbols) const;
	void composeTransitionInputSymbol(std::list<sax::Token>&, const alphabet::Symbol& symbol) const;
	void composeTransitionInputEpsilonSymbol(std::list<sax::Token>&, const std::variant<string::Epsilon, alphabet::Symbol>&) const;
	void composeTransitionOutputSymbol(std::list<sax::Token>&, const alphabet::Symbol& symbol) const;
	void composeTransitionOutputEpsilonSymbol(std::list<sax::Token>&, const std::variant<string::Epsilon, alphabet::Symbol>&) const;

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	std::list<sax::Token> compose(const AutomatonBase& automaton) const;

	std::list<sax::Token> compose(const Automaton& automaton) const;
	
	std::list<sax::Token> compose(const UnknownAutomaton& automaton) const;
	std::list<sax::Token> compose(const DFA& automaton) const;
	std::list<sax::Token> compose(const NFA& automaton) const;
	std::list<sax::Token> compose(const EpsilonNFA& automaton) const;
	std::list<sax::Token> compose(const ExtendedNFA& automaton) const;
	std::list<sax::Token> compose(const CompactNFA& automaton) const;
	std::list<sax::Token> compose(const PDA& automaton) const;
	std::list<sax::Token> compose(const OneTapeDTM& automaton) const;
};

} /* namespace automaton */

#endif /* AUTOMATON_TO_XML_COMPOSER_H_ */

