#include "FiniteAutomatonFromStringLexer.h"

namespace automaton {

FiniteAutomatonFromStringLexer::FiniteAutomatonFromStringLexer(std::stringstream& in) : m_In(in) {
	m_Current.type = TokenType::ERROR;
	m_Current.value = "";
}

FiniteAutomatonFromStringLexer& FiniteAutomatonFromStringLexer::next() {
	char character;
	m_Current.value = "";

	std::streampos pos = m_In.tellg();
L0:
	character = m_In.get();
	if(m_In.eof()) {
		m_In.seekg(pos);
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == ' ' || character == '\t') {
		goto L0;
	} else if(character == '\n') {
		m_Current.type = TokenType::NEW_LINE;
		m_Current.value += character;
		return *this;
	} else if(character == '<') {
		m_Current.type = TokenType::OUT;
		m_Current.value += character;
		return *this;
	} else if(character == '>') {
		m_Current.type = TokenType::IN;
		m_Current.value += character;
		return *this;
	} else if(character == '|') {
		m_Current.type = TokenType::SEPARATOR;
		m_Current.value += character;
		return *this;
	} else if(character == '-') {
		m_Current.type = TokenType::NONE;
		m_Current.value += character;
		return *this;
	} else if(character == '#') {
		m_Current.value += character;
		goto L1;
	} else if(m_In.seekg(pos), m_In >> "ENFA") {
		m_Current.type = TokenType::EPSILON_NFA;
		m_Current.value = "ENFA";
		return *this;
	} else if(m_In.clear(), m_In >> "NFA") {
		m_Current.type = TokenType::NFA;
		m_Current.value = "NFA";
		return *this;
	} else if(m_In.clear(), m_In >> "DFA") {
		m_Current.type = TokenType::DFA;
		m_Current.value = "DFA";
		return *this;
	} else {
		m_In.clear();
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	}
L1:
	character = m_In.get();
	if(m_In.eof()) {
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	} else if(character == 'E') {
		m_Current.value += character;
		m_Current.type = TokenType::EPSILON;
		return *this;
	} else {
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	}
}

FiniteAutomatonFromStringLexer::Token FiniteAutomatonFromStringLexer::token() {
	return m_Current;
}

} /* namespace automaton */
