/*
 * UnknownTransition.cpp
 *
 *  Created on: Oct 13, 2013
 *      Author: martin
 */

#include "UnknownTransition.h"
#include "../std/vector.hpp"

namespace automaton {

UnknownTransition::UnknownTransition() : from(NULL), to(NULL), input(NULL), output(NULL), shift(NOT_SET) {

}

UnknownTransition::UnknownTransition(const State& from, const State& to, const std::vector<alphabet::Symbol>& pop, const std::vector<alphabet::Symbol>& push, const std::variant<string::Epsilon, alphabet::Symbol>& input, const std::variant<string::Epsilon, alphabet::Symbol>& output, Shift shift) :
		from(new State(from)), to(new State(to)), pop(pop), push(push), input(new std::variant<string::Epsilon, alphabet::Symbol>(input)), output(new std::variant<string::Epsilon, alphabet::Symbol>(output)), shift(shift) {

}

UnknownTransition::~UnknownTransition() {

}

void UnknownTransition::setFrom(const State& state) {
	delete from;
	from = new State(state);
}

bool UnknownTransition::hasFrom() const {
	return from != NULL;
}

void UnknownTransition::clearFrom() {
	delete from;
	from = NULL;
}

const State& UnknownTransition::getFrom() const {
	return *from;
}

void UnknownTransition::setTo(const State& state) {
	delete to;
	to = new State(state);
}

bool UnknownTransition::hasTo() const {
	return to != NULL;
}

void UnknownTransition::clearTo() {
	delete to;
	to = NULL;
}

const State& UnknownTransition::getTo() const {
	return *to;
}

void UnknownTransition::setPop(const std::vector<alphabet::Symbol>& pop) {
	this->pop = pop;
}

const std::vector<alphabet::Symbol>& UnknownTransition::getPop() const {
	return pop;
}

void UnknownTransition::addPop(const alphabet::Symbol& symbol) {
	pop.push_back(symbol);
}

void UnknownTransition::setPush(const std::vector<alphabet::Symbol>& push) {
	this->push = push;
}

const std::vector<alphabet::Symbol>& UnknownTransition::getPush() const {
	return push;
}

void UnknownTransition::addPush(const alphabet::Symbol& symbol) {
	push.push_back(symbol);
}

void UnknownTransition::setOutput(const std::variant<string::Epsilon, alphabet::Symbol>& symbol) {
	delete output;
	output = new std::variant<string::Epsilon, alphabet::Symbol>(symbol);
}

bool UnknownTransition::hasOutput() const {
	return output != NULL;
}

void UnknownTransition::clearOutput() {
	delete output;
	output = NULL;
}

const std::variant<string::Epsilon, alphabet::Symbol>& UnknownTransition::getOutput() const {
	return *output;
}

void UnknownTransition::setInput(const std::variant<string::Epsilon, alphabet::Symbol>& symbol) {
	delete input;
	input = new std::variant<string::Epsilon, alphabet::Symbol>(symbol);
}

bool UnknownTransition::hasInput() const {
	return input != NULL;
}

void UnknownTransition::clearInput() {
	delete input;
	input = NULL;
}

const std::variant<string::Epsilon, alphabet::Symbol>& UnknownTransition::getInput() const {
	return *input;
}

const Shift& UnknownTransition::getShift() const {
	return shift;
}

void UnknownTransition::setShift(const Shift& shift) {
	this->shift = shift;
}

bool UnknownTransition::operator <(const UnknownTransition& other) const {
	if (from != other.from && from != NULL && other.from != NULL && *from != *other.from) {
		if(from == NULL) return -1;
		else if(other.from == NULL) return 1;
		else return *from < *other.from;
	} else if (input != other.input && input != NULL && other.input != NULL && *input != *other.input) {
		if(input == NULL) return -1;
		else if(other.input == NULL) return 1;
		else return *input < *other.input;
	} else if (pop != other.pop) {
		return pop < other.pop;
	} else if (shift != other.shift) {
		return shift < other.shift;
	} else if (to != other.to && to != NULL && other.to != NULL && *to != *other.to) {
		if(to == NULL) return 1;
		else if(other.to == NULL) return -1;
		else return *to < *other.to;
	} else if (output != other.output && output != NULL && other.output != NULL && *output != *other.output) {
		if(output == NULL) return 1;
		else if(other.output == NULL) return -1;
		else return *output < *other.output;
	} else {
		return push < other.push;
	}
}

bool UnknownTransition::operator ==(const UnknownTransition& other) const {
	return (from == other.from || (from != NULL && other.from != NULL && *from == *other.from))
		&& (input == other.input || (input != NULL && other.input != NULL && *input == *other.input))
		&& pop == other.pop
		&& shift == other.shift
		&& (to == other.to || (to != NULL && other.to != NULL && *to == *other.to))
		&& (output == other.output || (output != NULL && other.output != NULL && *output == *other.output))
		&& push == other.push;
}

bool UnknownTransition::operator !=(const UnknownTransition& other) const {
	return !(*this == other);
}

std::ostream& operator<<(std::ostream& out, const UnknownTransition& transition) {
	out << "(UnknownTransition" << " from = ";
	if(transition.from == NULL) out << "NULL"; else out << *transition.from;
	out << " input = ";
	if(transition.input == NULL) out << "NULL"; else out << *transition.input;
	out << " to = ";
	if(transition.to == NULL) out << "NULL"; else out << *transition.to;
	out << " output = ";
	if(transition.output == NULL) out << "NULL"; else out << *transition.output;
	out << " pop = " << transition.pop
		<< " push = " << transition.push
		<< " shift = " << SHIFT_NAMES[transition.shift]
		<< ")";

	return out;
}

} /* namespace automaton */
