/*
 * LabelFactory.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#ifndef LABEL_FACTORY_H_
#define LABEL_FACTORY_H_

#include <string>
#include <list>
#include "../sax/Token.h"
#include "../label/Label.h"

namespace label {

/**
 * String builder.
 */
class LabelFactory {
public:
	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static Label fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static Label fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static Label fromStdin();
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static Label fromStream(std::istream& in);

	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static void toFile(const Label&, const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static std::string toString(const Label&);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static void toStdout(const Label&);
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static void toStream(const Label&, std::ostream& out);

	
protected:
	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static Label parse(std::list<sax::Token> tokens);

	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static std::list<sax::Token> compose(const Label&);
};

} /* namespace label */

#endif /* LABEL_FACTORY_H_ */
