/*
 * StringFactory.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#ifndef STRING_FACTORY_H_
#define STRING_FACTORY_H_

#include <string>
#include <list>
#include "../sax/Token.h"
#include "../string/String.h"

namespace string {

/**
 * String builder.
 */
class StringFactory {
public:
	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static String fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static String fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static String fromStdin();
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static String fromStream(std::istream& in);

	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static void toFile(const String&, const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static std::string toString(const String&);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static void toStdout(const String&);
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static void toStream(const String&, std::ostream& out);

	
protected:
	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static String parse(std::list<sax::Token> tokens);

	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static std::list<sax::Token> compose(const String&);
};

} /* namespace string */

#endif /* STRING_FACTORY_H_ */
