/*
 * SymbolFactory.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_FACTORY_H_
#define SYMBOL_FACTORY_H_

#include <string>
#include <list>
#include "../sax/Token.h"
#include "../alphabet/Symbol.h"

namespace alphabet {

/**
 * String builder.
 */
class SymbolFactory {
public:
	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static Symbol fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static Symbol fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static Symbol fromStdin();
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static Symbol fromStream(std::istream& in);

	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static void toFile(const Symbol&, const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static std::string toString(const Symbol&);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static void toStdout(const Symbol&);
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static void toStream(const Symbol&, std::ostream& out);

	
protected:
	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static Symbol parse(std::list<sax::Token> tokens);

	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static std::list<sax::Token> compose(const Symbol&);
};

} /* namespace alphabet */

#endif /* SYMBOL_FACTORY_H_ */
