#include "LabelFromStringParser.h"
#include "../exception/AlibException.h"
#include "StringLabel.h"
#include "IntegerLabel.h"
#include "CharacterLabel.h"
#include <algorithm>

namespace label {

LabelFromStringParser::LabelFromStringParser(std::stringstream& input) : m_Lexer(input) {

}

Label LabelFromStringParser::parse() {
	return parse(std::set<FEATURES>({FEATURES::STRING, FEATURES::CHAR, FEATURES::INTEGER}));
}

Label LabelFromStringParser::parse(const std::set<FEATURES>& features) {
	LabelFromStringLexer::Token token = m_Lexer.token();
	switch(token.type) {
	case LabelFromStringLexer::TokenType::TEOF:
		throw exception::AlibException();
	case LabelFromStringLexer::TokenType::STRING:
		if(!features.count(FEATURES::STRING)) throw exception::AlibException();
		return Label(label::StringLabel(token.value));
	case LabelFromStringLexer::TokenType::CHAR:
		if(!features.count(FEATURES::CHAR)) throw exception::AlibException();
		return Label(label::CharacterLabel(token.value[0]));
	case LabelFromStringLexer::TokenType::INTEGER:
		if(!features.count(FEATURES::INTEGER)) throw exception::AlibException();
		return Label(label::IntegerLabel(std::stoi(token.value)));
	case LabelFromStringLexer::TokenType::ERROR:
		break;
	}
	throw exception::AlibException();
}

Label LabelFromStringParser::parseValue() {
	first();
	Label res = parse();
	
	LabelFromStringLexer::Token token = m_Lexer.next().token();
	if(token.type == LabelFromStringLexer::TokenType::TEOF) {
		return std::move(res);
	} else {
		throw exception::AlibException();
	}
}

bool LabelFromStringParser::next() {
	LabelFromStringLexer::Token token = m_Lexer.next().token();
	if(token.type == LabelFromStringLexer::TokenType::STRING || token.type == LabelFromStringLexer::TokenType::CHAR || token.type == LabelFromStringLexer::TokenType::INTEGER) {
		return true;
	} else {
		return false;
	}
}

bool LabelFromStringParser::first() {
	LabelFromStringLexer::Token token = m_Lexer.next().token();
	if(token.type == LabelFromStringLexer::TokenType::STRING || token.type == LabelFromStringLexer::TokenType::CHAR || token.type == LabelFromStringLexer::TokenType::INTEGER) {
		return true;
	} else {
		return false;
	}
}

Label* LabelFromStringParser::parsePointer() {
	if(first()) {
		return new Label(parse());
	} else {
		return NULL;
	}
}

} /* namespace label */
